<?php
namespace Elementor;
use Elementor\Icons_Manager;

// If this file is called directly, abort.
if (!defined('ABSPATH')) {
    exit;   
}

class WOOLISTING_CATEGORY extends Widget_Base {

    // Function for get the slug of the element name.
    public function get_name() {
        return 'woolisting-category';
    }
    // Function for get the name of the element.
    public function get_title() {
        return esc_html__('WooListing Category', 'woolisting');
    }

    // Function for get the icon of the element.
    public function get_icon() {
        return  'eicon-product-categories';
    }

    // Function for include element into the category.
    public function get_categories() {
        return ['woolisting'];
    }

    //Adding the controls fields for Woolisting Element.
    protected function register_controls() {

        // Start Woolisting Category Layout Section.
        $this->start_controls_section(
            'woolist_general_section', array(
                'label'             => esc_html__('Category Layout', 'woolisting'),
            )
        );

        // Layout Style.
        $this->add_control(
            'woolisting_category_layout', [
            'label'                 => __('Layout Style', 'woolisting'),
            'type'                  => Controls_Manager::SELECT,
            'options'               => [
                'grid'              => esc_html__('Grid', 'woolisting'),
                'slider'            => esc_html__('Slider', 'woolisting'),
            ],
            'default'               => 'grid',
            ]
        );

        // Layout Design.
        $this->add_control(
            'woolisting_category_style', [
            'label'                 => esc_html__('Layouts', 'woolisting'),
            'type'                  => Controls_Manager::SELECT,
            'options'               => [
                'woolisting-category-style-1'  => esc_html__('Style 1', 'woolisting'),
                'woolisting-category-style-2'  => esc_html__('Style 2', 'woolisting'),
                'woolisting-category-style-3'  => esc_html__('Style 3', 'woolisting'),
                'woolisting-category-style-4'  => esc_html__('Style 4', 'woolisting'),
            ],
            'default'               => 'woolisting-category-style-1',
            ]
        );

        //No.of Category.
        $this->add_control(
            'woolisting_number_of_category', [
                'label'             => esc_html__('Display No. of Category', 'woolisting'),
                'type'              => Controls_Manager::NUMBER,
                'min'               => 1,
                'max'               => 100,
                'step'              => 1,
                'default'           => 5,
            ]
        );

        // Grid Columns.
        $this->add_responsive_control(
            'woolist_grid_columns', [
                'label'             => esc_html__('Grid Columns', 'woolisting'),
                'type'              => Controls_Manager::SELECT,
                'default'           => '3',
                'tablet_default'    => '2',
                'mobile_default'    => '1',
                'options'           => [
                    '1'             => '1',
                    '2'             => '2',
                    '3'             => '3',
                    '4'             => '4',
                    '5'             => '5',
                    '6'             => '6',
                ],  
                'prefix_class'      => 'elementor-grid%s-',
                'frontend_available'=> true,
                'selectors'         => [
                    '.elementor-msie {{WRAPPER}} .elementor-product-item' => 'width: calc(100% / {{SIZE}} )',
                ],
                'condition'         => ['woolisting_category_layout' => 'grid'],
            ]
        );

        // Show Image.
        $this->add_control(
            'woolisting_show_image', [
                'label'             => esc_html__('Show Category Image', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', 'woolisting'),
                'label_off'         => esc_html__('Hide', 'woolisting'),
                'default'           => 'yes',
                'separator'         => 'before',
            ]
        );

        // Show Title.
            $this->add_control(
            'woolisting_show_title', [
                'label'             => esc_html__('Show Category Title', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_off'         => esc_html__('Show', 'woolisting'),
                'label_on'          => esc_html__('Hide', 'woolisting'),
                'default'           => 'yes',
            ]
        );

        // Show Description.
        $this->add_control(
            'woolisting_show_description', [
                'label'             => esc_html__('Show Description', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_off'         => esc_html__('Show', 'woolisting'),
                'label_on'          => esc_html__('Hide', 'woolisting'),
                'default'           => 'yes',
            ]
        );

        // Content Length.
        $this->add_control(
            'woolisting_description_length', [
                'label'             => esc_html__('Description Length', 'woolisting'),
                'type'              => Controls_Manager::NUMBER,
                'default'           => apply_filters('woolisting_content_length', 15),
                'condition'         => [
                    'woolisting_show_description'  => 'yes',
                ],
            ]
        );

        // Show Post Count.
        $this->add_control(
            'woolisting_show_post_count', [
                'label'             => esc_html__('Show Post Count', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_off'         => esc_html__('Show', 'woolisting'),
                'label_on'          => esc_html__('Hide', 'woolisting'),
                'default'           => 'yes',
            ]
        );

        // Show Post Text.
        $this->add_control(
            'woolisting_post_text', [
                'label'             => esc_html__('One Post', 'woolisting'),
                'type'              => Controls_Manager::TEXT,
                'default'           => esc_html__('Post', 'woolisting'),
            ]
        );

        // Show Post Text.
        $this->add_control(
            'woolisting_more_post_text', [
                'label'             => esc_html__('More Than One Post', 'woolisting'),
                'type'              => Controls_Manager::TEXT,
                'default'           => esc_html__('Posts', 'woolisting'),
            ]
        );

        // Post Order By.
        $this->add_control(
            'woolisting_category_orderby', [
                'label'             => esc_html__('Order By', 'woolisting'),
                'type'              => Controls_Manager::SELECT,
                'default'           => 'title',
                'options'           => [
                'date'              => esc_html__('Date', 'woolisting'),
                'title'             => esc_html__('Name', 'woolisting'),
                'menu_order'        => esc_html__('Menu Order', 'woolisting'),
                'rand'              => esc_html__('Random', 'woolisting'),
                'ID'                => esc_html__('ID', 'woolisting'),
                ],
            ]
        );

        // Post Sort By.
        $this->add_control(
            'woolisting_category_order', [
                'label'             => esc_html__('Sort By', 'woolisting'),
                'type'              => Controls_Manager::SELECT,
                'default'           => 'desc',
                'options'           => [
                    'asc'           => esc_html__('ASC', 'woolisting'),
                    'desc'          => esc_html__('DESC', 'woolisting'),
                ],
            ]
        );

        $this->end_controls_section();
        // End Woolisting Category Layout Section.

        // Grid Configuration.
        $this->start_controls_section(
            'woolisting_grid_options', [
                'label'             => esc_html__('Grid Configuration', 'woolisting'),
                'tab'               => Controls_Manager::TAB_CONTENT,
                'condition'         => ['woolisting_category_layout' => 'grid'],
            ]
        );

        // Grid Column Gap.
        $this->add_responsive_control(
            'woolisting_product_column_gap', [
                'label'             => esc_html__('Grid Column Gap', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'default'           => [
                    'size'          => 30,
                ],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available'  => true,
                'selectors'           => [
                    '{{WRAPPER}} .product-grid-gap' => 'grid-column-gap: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Grid Row Gap.
        $this->add_responsive_control(
            'woolisting_product_row_gap', [
                'label'             => esc_html__('Grid Row Gap', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'default'           => [
                    'size'          => 30,
                ],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available'  => true,
                'selectors'           => [
                    '{{WRAPPER}} .product-grid-gap' => 'grid-row-gap: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        // End Grid Configuration.

        // Start Slider Configuration.
        $this->start_controls_section(
            'woolisting_slider_options', [
                'label'             => esc_html__('Slider Configuration', 'woolisting'),
                'tab'               => Controls_Manager::TAB_CONTENT,
                'condition'         => ['woolisting_category_layout' => 'slider'],
            ]
        );

        // Slide to show.
        $this->add_responsive_control(
            'woolisting_slider_slides_to_show', [
                'label'             => esc_html__('Slides To Show', 'woolisting'),
                'type'              => Controls_Manager::NUMBER,
                'default'           => 3,
                'devices'           => ['desktop', 'tablet', 'mobile'],
                'desktop_default'   => 3,
                'tablet_default'    => 2,
                'mobile_default'    => 1,
                'frontend_available'  => true,
            ]
        );

        // Slide to Scroll.
        $this->add_control(
            'woolisting_slider_slides_to_scroll', [
                'label'             => esc_html__('Slides To Scroll', 'woolisting'),
                'type'              => Controls_Manager::NUMBER,
                'min'               => 1,
                'max'               => 6,
                'step'              => 1,
                'default'           => 1,
            ]
        );

        // Slider Infinite.
        $this->add_control(
            'woolisting_slider_infinite', [
                'label'             => esc_html__('Show Infinite', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', 'woolisting'),
                'label_off'         => esc_html__('Hide', 'woolisting'),
                'return_value'      => 'yes',
                'default'           => 'no',
            ]
        );

        // Pause On Hover.
        $this->add_control(
            'woolisting_slider_pause_on_hover', [
                'label'            => esc_html__('Pause On Hover', 'woolisting'),
                'type'             => Controls_Manager::SWITCHER,
                'label_on'         => esc_html__('Show', 'woolisting'),
                'label_off'        => esc_html__('Hide', 'woolisting'),
                'return_value'     => 'yes',
                'default'          => 'no',
            ]
        );

        // Slider Autoplay.
        $this->add_control(
            'woolisting_slider_autoplay', [
                'label'             => esc_html__('Autoplay', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'default'           => 'yes',
                'label_off'         => esc_html__('Off', 'woolisting'),
                'label_on'          => esc_html__('On', 'woolisting'),
            ]
        );

        // Autoplay Speed.
        $this->add_control(
            'woolisting_slider_speed', [
                'label'             => esc_html__('Autoplay Speed', 'woolisting'),
                'type'              => Controls_Manager::NUMBER,
                'min'               => 1000,
                'max'               => 10000,
                'step'              => 1000,
                'default'           => 5000,
                'description'       => esc_html__('Speed is in milliseconds (1000ms = 1s).', 'woolisting')
            ]
        );

        // Show Dots.
        $this->add_control(
            'woolisting_slider_dots', [
                'label'             => esc_html__('Show Dots', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', 'woolisting'),
                'label_off'         => esc_html__('Hide', 'woolisting'),
                'return_value'      => 'yes',
                'default'           => 'no',
            ]
        );

        // Show Arrows.
        $this->add_control(
            'woolisting_slider_arrows', [
                'label'            => esc_html__('Show Arrows', 'woolisting'),
                'type'             => Controls_Manager::SWITCHER,
                'label_on'         => esc_html__('Show', 'woolisting'),
                'label_off'        => esc_html__('Hide', 'woolisting'),
                'return_value'     => 'yes',
                'default'          => 'yes',
            ]
        );

        // Previous Icon.
        $this->add_control(
            'woolisting_prev_icon', [
                'label'            => esc_html__('Previous icon', 'woolisting'),
                'type'             => Controls_Manager::ICONS,
                'default'          => [
                    'value'        => 'fas fa-chevron-left',
                    'library'      => 'solid',
                ],
            ]
        );

        // Next Icon.
        $this->add_control(
            'woolisting_next_icon', [
                'label'            => esc_html__('Next icon', 'woolisting'),
                'type'             => Controls_Manager::ICONS,
                'default'          => [
                    'value'        => 'fas fa-chevron-right',
                    'library'      => 'solid',
                ]
            ]
        );

        // Space Between.
        $this->add_responsive_control(
            'woolisting_slider_gap', [
                'label'             => esc_html__('Column Gap', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'default'           => [
                    'size'          => 10,
                ],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available'  => true,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-slider .woolisting-category-img ' => 'margin-left: {{SIZE}}{{UNIT}}; margin-right: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section(); 
        // End Slider Configuration.

        // Start Item Style.
        $this->start_controls_section(
            'woolist_item_style', [
                'label'            => esc_html__('Item', 'woolisting'),
                'tab'              => Controls_Manager::TAB_STYLE,
            ]
        );

        // Start Item Tab.
        $this->start_controls_tabs('woolist_color_tabs');

        // Start Normal Item Tab.
        $this->start_controls_tab('woolist_normal_item', [
            'label'                => esc_html__('Normal', 'woolisting'),
            ]
        );

        // Item Background Color.
        $this->add_control(
            'woolist_item_bg_color', [
                'label'            => esc_html__('Content Background Color', 'woolisting'),
                'type'             => Controls_Manager::COLOR,
                'selectors'        => [
                    '{{WRAPPER}} .woolist-category .woolist-category-content,
                    {{WRAPPER}} .woolisting-category-style-3 .woolist-content' => 'background-color: {{VALUE}};',
                ],
                'condition'        => [
                    'woolisting_category_style'   => ['woolisting-category-style-1', 'woolisting-category-style-3'],
                ],
            ]
        );

        // Item Box Shadow.
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(), [
                'name'             => 'woolist_item_box_shadow',
                'selector'         => '{{WRAPPER}} .woolist-category .woolist-category-img',
            ]
        );

        $this->end_controls_tab();
        // End Normal Item Tab.

        // Start Hover Item Tab.
        $this->start_controls_tab('woolist_hover_item', [
            'label'                => esc_html__('Hover', 'woolisting'),
            ]
        );

        // Item Hover Background Color.
        $this->add_control(
            'woolist_item_bg_color_hover', [
                'label'            => esc_html__('Content Background Color', 'woolisting'),
                'type'             => Controls_Manager::COLOR,
                'selectors'        => [
                    '{{WRAPPER}} .woolist-category:hover .woolist-category-content,
                    {{WRAPPER}} .woolisting-category-style-3 .woolist-category:hover .woolist-content' => 'background-color: {{VALUE}};',
                ],
                'condition'        => [
                    'woolisting_category_style'   => ['woolisting-category-style-1', 'woolisting-category-style-3'],
                ],
            ]
        );

        // Item Hover Border.
        $this->add_control(
            'woolist_border_color_hover', [
                'label'            => esc_html__('Border Color', 'woolisting'),
                'type'             => Controls_Manager::COLOR,
                'selectors'        => [
                    '{{WRAPPER}} .woolist-category:hover .woolist-category-img' => 'border-color: {{VALUE}};',
                ],
            ]
        );

        // Item Box Shadow.
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(), [
                'name'             => 'woolist_item_hover_box_shadow',
                'selector'         => '{{WRAPPER}} .woolist-category:hover .woolist-category-img',
            ]
        );

        $this->end_controls_tab();
        // End Hover Item Tab.

        $this->end_controls_tabs();
        // End Item Style.

        // Item border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'             => 'woolist_item_border',
                'selector'         => '{{WRAPPER}} .woolisting-category-style-1 .woolist-category-img,
                {{WRAPPER}} .woolisting-category-style-2 .woolist-frame,
                {{WRAPPER}} .woolisting-category-style-3 .woolist-category-img,
                {{WRAPPER}} .woolisting-category-style-4 .woolist-category-img,
                {{WRAPPER}} .woolisting-category-style-5 .woolist-category-img',
            ]
        );

        // Item Border Radius.
        $this->add_responsive_control(
            'woolist_border_radius', [
                'label'            => esc_html__('Border Radius', 'woolisting'),
                'type'             => Controls_Manager::DIMENSIONS,
                'size_units'       => ['px', '%'],
                'selectors'        => [
                    '{{WRAPPER}} .woolist-category .woolist-category-img,
                    {{WRAPPER}} .woolisting-category-style-2 .woolist-frame,
                    {{WRAPPER}} .woolisting-category-style-2 .woolist-category-content,
                    {{WRAPPER}} .woolisting-category-style-5 .overlay' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        // Content Border Radius.
        $this->add_responsive_control(
            'woolist_content_radius', [
                'label'            => esc_html__('Content Border Radius', 'woolisting'),
                'type'             => Controls_Manager::DIMENSIONS,
                'size_units'       => ['px', '%'],
                'selectors'        => [
                    '{{WRAPPER}} .woolisting-category-style-3 .woolist-content' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'        => [
                    'woolisting_category_style'   => ['woolisting-category-style-3'],
                ],
            ]
        );

        // Item Padding.
        $this->add_responsive_control(
            'woolist_item_padding', [
                'label'            => esc_html__('Padding', 'woolisting'),
                'type'             => Controls_Manager::DIMENSIONS,
                'size_units'       => ['px', '%'],
                'selectors'        => [
                    '{{WRAPPER}} .woolisting-category-style-1 .woolist-category-content,
                    {{WRAPPER}} .woolisting-category-style-2 .woolist-content,
                    {{WRAPPER}} .woolisting-category-style-3 .woolist-content,
                    {{WRAPPER}} .woolisting-category-style-4 .woolist-content,
                    {{WRAPPER}} .woolisting-category-style-5 .woolist-content' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        // Item Margin.
        $this->add_responsive_control(
            'woolist_item_margin', [
                'label'            => esc_html__('Margin', 'woolisting'),
                'type'             => Controls_Manager::DIMENSIONS,
                'size_units'       => ['px', '%'],
                'selectors'        => [
                    '{{WRAPPER}} .woolisting-category-style-3 .woolist-content' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'        => [
                    'woolisting_category_style'   => ['woolisting-category-style-3'],
                ],
            ]
        );

        $this->end_controls_section();
        // End Item Style.

        // Start Image Style.
        $this->start_controls_section(
            'woolist_imge_style', [
                'label'            => esc_html__('Image', 'woolisting'),
                'tab'              => Controls_Manager::TAB_STYLE,
            ]
        );

        // Image Width.
        $this->add_responsive_control(
            'woolist_image_width', [
                'label'            => esc_html__('Image Width', 'woolisting'),
                'type'             => Controls_Manager::SLIDER,
                'size_units'       => ['px', '%'],
                'range'            => [
                    'px'           => [
                        'min'      => 0,
                        'max'      => 800,
                        'step'     => 1,
                    ],
                    '%'            => [
                        'min'      => 0,
                        'max'      => 100,
                    ],
                ],
                'default'          => [
                    'size'         => 100,
                    'unit'         => '%',
                ],
                'selectors'        => [
                    '{{WRAPPER}} .woolist-category .product-category img' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Image Height.
        $this->add_responsive_control(
            'woolist_image_height', [
                'label'            => esc_html__('Image Height', 'woolisting'),
                'type'             => Controls_Manager::SLIDER,
                'size_units'       => ['px', '%'],
                'range'            => [
                    'px'           => [
                        'min'      => 0,
                        'max'      => 800,
                        'step'     => 1,
                    ],
                    '%'            => [
                        'min'      => 0,
                        'max'      => 100,
                    ],
                ],
                'selectors'        => [
                    '{{WRAPPER}} .woolist-category .product-category img' => 'height: {{SIZE}}{{UNIT}} !important;',
                ],
            ]
        );

        // Image Border Radius.
        $this->add_responsive_control(
            'woolist_image_border_radius', [
                'label'             => esc_html__('Border Radius', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => ['px', '%'],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-category .product-category img,
                    {{WRAPPER}} .woolisting-category-style-1 .product-category::after' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolisting_category_style'   => 'woolisting-category-style-1',
                ],
            ]
        );

        // Image Overlay.
        $this->add_control(
            'woolist_overlay_color', [
                'label'            => esc_html__('Image Overlay', 'woolisting'),
                'type'             => Controls_Manager::COLOR,
                'selectors'        => [
                    '{{WRAPPER}} .woolisting-category-style-1 .product-category::after,
                    {{WRAPPER}} .woolisting-category-style-2 .product-category::after,
                    {{WRAPPER}} .woolisting-category-style-3 .product-category::after,
                    {{WRAPPER}} .woolisting-category-style-4 .product-category::after,
                    {{WRAPPER}} .woolisting-category-style-5 .woolist-category-img:hover .overlay' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        // Image Overlay Opacity.
        $this->add_control(
            'woolist_overlay_opacity', [
                'label'             => esc_html__('Overlay Opacity', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'max'       => 1,
                        'step'      => 0.01,
                    ],
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-style-1 .product-category:hover:after,
                    {{WRAPPER}} .woolisting-category-style-2 .woolist-category-img:hover .product-category::after,
                    {{WRAPPER}} .woolisting-category-style-3 .product-category::after,
                    {{WRAPPER}} .woolisting-category-style-4 .product-category::after,
                    {{WRAPPER}} .woolisting-category-style-5 .woolist-category-img:hover .overlay' => 'opacity: {{SIZE}};',
                ],
            ]
        );

        $this->end_controls_section();
        // End Image Style.

        // Start Title Style.
        $this->start_controls_section(
            'woolist_imge', [
                'label'            => esc_html__('Title', 'woolisting'),
                'tab'              => Controls_Manager::TAB_STYLE,
            ]
        );

        // Title Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'             => 'woolist_title_typography',
                'label'            => esc_html__('Typography', 'woolisting'),
                'selector'         => '{{WRAPPER}} .woolist-category .woolist-category-title h2',
            ]
        );

        // Title Color.
        $this->add_control(
            'woolist_title_color', [
                'label'            => esc_html__('Color', 'woolisting'),
                'type'             => Controls_Manager::COLOR,
                'selectors'        => [
                    '{{WRAPPER}} .woolist-category .woolist-category-title h2' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Title Spacing.
        $this->add_responsive_control(
            'woolist_title_spacing', [
                'label'             => esc_html__('Title Spacing', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available'  => true,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-style-2 .woolist-category-title h2' => 'margin-top: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .woolisting-category-style-1 .woolist-category-title h2,
                    {{WRAPPER}} .woolisting-category-style-3 .woolist-category-title h2,
                    {{WRAPPER}} .woolisting-category-style-4 .woolist-category-title h2' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Title Border Color.
        $this->add_control(
            'woolist_title_border_color', [
                'label'            => esc_html__('Border Color', 'woolisting'),
                'type'             => Controls_Manager::COLOR,
                'selectors'        => [
                    '{{WRAPPER}} .woolisting-category-style-4 .woolist-category-title h2:after' => 'border-color: {{VALUE}};',
                ],
                'condition'        => [
                    'woolisting_category_style'   => 'woolisting-category-style-4',
                ],
            ]
        );

        // Title Border Width.
        $this->add_responsive_control(
            'woolist_border_width', [
                'label'             => esc_html__('Border Width', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 10,
                    ],
                ],
                'frontend_available'  => true,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-style-4 .woolist-category-title h2:after' => 'border-width: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolisting_category_style'   => 'woolisting-category-style-4',
                ],
            ]
        );

        // Title Alignment.
        $this->add_responsive_control(
            'woolist_title_align', [
                'label'             => esc_html__('Alignment', 'woolisting'),
                'type'              => Controls_Manager::CHOOSE,
                'options'           => [
                    'left' 	        => [
                        'title'     => esc_html__('Left', 'woolisting'),
                        'icon'      => 'eicon-text-align-left',
                    ],
                    'center'        => [
                        'title'     => esc_html__('Center', 'woolisting'),
                        'icon'      => 'eicon-text-align-center',
                    ],
                    'right' 		=> [
                        'title' 	=> esc_html__('Right', 'woolisting'),
                        'icon'  	=> 'eicon-text-align-right',
                    ],
                ],
                'default'   		=> 'center',
                'selectors' 		=> [
                    '{{WRAPPER}} .woolist-category .woolist-category-title h2,
                    {{WRAPPER}} .woolisting-category-style-4 .woolist-category-title' => 'text-align: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_section();
        // End Title Style.

        // Start Description Style.
        $this->start_controls_section(
            'woolist_description_style', [
                'label'            => esc_html__('Description', 'woolisting'),
                'tab'              => Controls_Manager::TAB_STYLE,
            ]
        );

        // Description Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'             => 'woolist_description_typography',
                'label'            => esc_html__('Typography', 'woolisting'),
                'selector'         => '{{WRAPPER}} .woolist-category .woolist-category-description',
            ]
        );

        // Description Color.
        $this->add_control(
            'woolist_description_color', [
                'label'            => esc_html__('Color', 'woolisting'),
                'type'             => Controls_Manager::COLOR,
                'selectors'        => [
                    '{{WRAPPER}} .woolist-category .woolist-category-description' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Description Spacing.
        $this->add_responsive_control(
            'woolist_post_count_spacing', [
                'label'             => esc_html__('Description Spacing', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available'  => true,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-category .woolist-category-description' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Description Alignment.
        $this->add_responsive_control(
            'woolist_description_align', [
                'label'             => esc_html__('Alignment', 'woolisting'),
                'type'              => Controls_Manager::CHOOSE,
                'options'           => [
                    'left' 	        => [
                        'title'     => esc_html__('Left', 'woolisting'),
                        'icon'      => 'eicon-text-align-left',
                    ],
                    'center' 		=> [
                        'title' 	=> esc_html__('Center', 'woolisting'),
                        'icon'  	=> 'eicon-text-align-center',
                    ],
                    'right' 		=> [
                        'title' 	=> esc_html__('Right', 'woolisting'),
                        'icon'  	=> 'eicon-text-align-right',
                    ],
                ],
                'default'   		=> 'center',
                'selectors' 		=> [
                    '{{WRAPPER}} .woolist-category .woolist-category-description' => 'text-align: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_section();
        // End Description Style.

        // Start Post Count Style.
        $this->start_controls_section(
            'woolist_post_count_style', [
                'label'             => esc_html__('Post Count', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        // Post Count Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'              => 'woolist_post_count_typography',
                'label'             => esc_html__('Typography', 'woolisting'),
                'selector'          => '{{WRAPPER}} .woolist-category  .woolist-post-count a',
            ]
        );

        // Post Count Color.
        $this->add_control(
            'woolist_post_count_color', [
                'label'             => esc_html__('Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-category .woolist-post-count a' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Post Count Background Color.
        $this->add_control(
            'woolist_post_count_bg_color', [
                'label'             => esc_html__('Background Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-style-1  .woolist-post-count a' => 'background-color: {{VALUE}};',
                ],
                'condition'         => [
                    'woolisting_category_style'   => 'woolisting-category-style-1',
                ],
            ]
        );

        // Post Count border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woolist_count_border',
                'selector'          => '{{WRAPPER}} .woolist-category .woolist-post-count a',
                'condition'         => [
                    'woolisting_category_style'   => 'woolisting-category-style-1',
                ],
            ]
        );

        // Post Count Border Radius.
        $this->add_responsive_control(
            'woolist_count_border_radius', [
                'label'             => esc_html__('Border Radius', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => ['px', '%'],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-category .woolist-post-count a' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolisting_category_style'   => 'woolisting-category-style-1',
                ],
            ]
        );

        // Post Count Padding.
        $this->add_responsive_control(
            'woolist_count_padding', [
                'label'             => esc_html__('Padding', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => ['px', '%'],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-category .woolist-post-count a' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolisting_category_style'   => 'woolisting-category-style-1',
                ],
            ]
        );

        // Post Count Box Shadow.
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(), [
                'name'              => 'woolist_count_box_shadow',
                'selector'          => '{{WRAPPER}} .woolist-category .woolist-post-count a',
                'condition'         => [
                    'woolisting_category_style'   => 'woolisting-category-style-1',
                ],
            ]
        );

        // Post Count Alignment.
        $this->add_responsive_control(
            'woolist_post_count_align', [
                'label'             => esc_html__('Alignment', 'woolisting'),
                'type'              => Controls_Manager::CHOOSE,
                'options'           => [
                    'left' 	        => [
                        'title'     => esc_html__('Left', 'woolisting'),
                        'icon'      => 'eicon-text-align-left',
                    ],
                    'center'        => [
                        'title'     => esc_html__('Center', 'woolisting'),
                        'icon'      => 'eicon-text-align-center',
                    ],
                    'right' 		=> [
                        'title' 	=> esc_html__('Right', 'woolisting'),
                        'icon'  	=> 'eicon-text-align-right',
                    ],
                ],
                'default'   		=> 'center',
                'selectors' 		=> [
                    '{{WRAPPER}} .woolist-category .woolist-post-count' => 'text-align: {{VALUE}};',
                ],
                'condition'         => [
                    'woolisting_category_style'   => ['woolisting-category-style-2', 'woolisting-category-style-3', 'woolisting-category-style-4', 'woolisting-category-style-5'],
                ],
            ]
        );

        $this->end_controls_section();
        // End Post Count Style.

        // Start Pagination Style.
        $this->start_controls_section(
            'woolisting_pagination_style', [
                'label'             => esc_html__('Pagination', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
                'condition'         => ['woolisting_category_layout' => 'slider'],
            ]
        );

        // Pagination Color.
        $this->add_control(
            'woolisting_pagination_color', [
                'label'             => esc_html__('Pagination Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'default'           => '#FD5056',
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-slider .slick-dots li button::before' => 'color: {{VALUE}} !important;',
                    '{{WRAPPER}} .woolisting-category-slider.woolisting-category-style-3 .slick-dots li button::before,
                    {{WRAPPER}} .woolisting-category-slider.woolisting-category-style-5 .slick-dots li button::before' => 'background-color: {{VALUE}} !important;',
                ],
            ]
        );

        // Active Pagination Color.
        $this->add_control(
            'woolisting_active_pagination_color', [
                'label'             => esc_html__('Active Pagination Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'default'           => '#FD5056',
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-slider .slick-dots li.slick-active button::before' => 'color: {{VALUE}} !important;',
                ],
            ]
        );

        // Pagination Size.
        $this->add_responsive_control(
            'woolisting_pagination_size', [
                'label'             => esc_html__('Pagination Size', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => ['px', '%'],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                        'step'      => 1,
                    ],
                ],
                'default'           => [
                    'size'          => 40,
                    'unit'          => 'px',
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-slider .slick-dots li button::before' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolisting_category_style'   => ['woolisting-category-style-1', 'woolisting-category-style-2', 'woolisting-category-style-4'],
                ],
            ]
        );

        // Active Pagination Size.
        $this->add_responsive_control(
            'woolisting_active_pagination_size', [
                'label'             => esc_html__('Active Pagination Size', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => ['px', '%'],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                        'step'      => 1,
                    ],
                ],
                'default'           => [
                    'size'          => 40,
                    'unit'          => 'px',
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-slider .slick-dots li.slick-active button::before' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolisting_category_style'   => ['woolisting-category-style-1', 'woolisting-category-style-2', 'woolisting-category-style-4'],
                ],
            ]
        );

        // Pagination Size.
        $this->add_responsive_control(
            'woolisting_pagination_square', [
                'label'             => esc_html__('Pagination Size', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => ['px', '%'],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                        'step'      => 1,
                    ],
                ],
                'default'           => [
                    'size'          => 14,
                    'unit'          => 'px',
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-slider.woolisting-category-style-3 .slick-dots li button::before' => 'width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolisting_category_style'   => 'woolisting-category-style-3',
                ],
            ]
        );

        $this->end_controls_section();
        // End Pagination Style.

        // Start Navigation Style. 
        $this->start_controls_section(
            'woolisting_arrow_style', [
                'label'             => esc_html__('Arrows', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
                'condition'         => ['woolisting_category_layout' => 'slider'],
            ]
        );

        // Arrow Size.
        $this->add_responsive_control(
            'woolisting_arrow_size', [
                'label'             => esc_html__('Arrow Size', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => ['px', '%'],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                        'step'      => 1,
                    ],
                ],
                'default'           => [
                    'size'          => 18,
                    'unit'          => 'px',
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-slider .slick-prev i,
                    {{WRAPPER}} .woolisting-category-slider .slick-next i' => 'font-size: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .woolisting-category-slider .slick-prev svg,
                    {{WRAPPER}} .woolisting-category-slider .slick-next svg' => 'width:{{SIZE}}{{UNIT}}; height:{{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Start Item Tab.
        $this->start_controls_tabs('woolisting_arrow_tabs');

        // Start Normal Item Tab.
        $this->start_controls_tab('woolisting_normal_arrow', [
            'label'                => esc_html__('Normal', 'woolisting'),
            ]
        );

        // Arrow Color.
        $this->add_control(
            'woolisting_arrow_color', [
                'label'             => esc_html__('Arrow Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-slider .slick-prev i,
                    {{WRAPPER}} .woolisting-category-slider .slick-next i' => 'color: {{VALUE}} !important;',
                    '{{WRAPPER}} .woolisting-category-slider .slick-prev svg,
                    {{WRAPPER}} .woolisting-category-slider .slick-next svg' => 'fill:{{VALUE}} !important;',
                ],
            ]
        );

        // Arrow Background Color.
        $this->add_control(
            'woolisting_arrow_background_color', [
                'label'             => esc_html__('Arrow Background Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-slider .slick-prev i,
                    {{WRAPPER}} .woolisting-category-slider .slick-next i' => 'background-color: {{VALUE}} !important;',
                ],
            ]
        );

        $this->end_controls_tab();
        // End Normal Item Tab.

        // Start Hover Item Tab.
        $this->start_controls_tab('woolisting_hover_arrow', [
            'label'                => esc_html__('Hover', 'woolisting'),
            ]
        );

        // Arrow Color.
        $this->add_control(
            'woolisting_arrow_color_hover', [
                'label'             => esc_html__('Arrow Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-slider .slick-prev i:hover,
                    {{WRAPPER}} .woolisting-category-slider .slick-next i:hover' => 'color: {{VALUE}} !important;',
                    '{{WRAPPER}} .woolisting-category-slider .slick-prev svg:hover,
                    {{WRAPPER}} .woolisting-category-slider .slick-next svg:hover' => 'fill:{{VALUE}} !important;',
                ],
            ]
        );

        // Arrow Background Color.
        $this->add_control(
            'woolisting_arrow_bg_hover_color', [
                'label'             => esc_html__('Arrow Background Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-slider .slick-prev i:hover,
                    {{WRAPPER}} .woolisting-category-slider .slick-next i:hover' => 'background-color: {{VALUE}} !important;',
                ],
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();
        // End Hover Item Tab.

        // Arrow border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'             => 'woolisting_arrow_border',
                'selector'         => '{{WRAPPER}} .woolisting-category-slider .slick-prev i,
                {{WRAPPER}} .woolisting-category-slider .slick-next i',
            ]
        );

        // Arrow Border Radius.
        $this->add_responsive_control(
            'woolisting_arrow_border_radius', [
                'label'             => esc_html__('Arrow Border Radius', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => ['px', '%'],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-slider .slick-prev i,
                    {{WRAPPER}} .woolisting-category-slider .slick-next i' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        // Arrow Padding.
        $this->add_responsive_control(
            'woolisting_arrow_padding', [
                'label'             => esc_html__('Arrow Padding', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => ['px', '%'],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-category-slider .slick-prev i,
                    {{WRAPPER}} .woolisting-category-slider .slick-next i' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        // End Navigation Style.

    }
    // End Woolisting Category Layout Section.

    protected function render() {

        $settings = $this->get_settings_for_display();

        $category_layout = $settings['woolisting_category_layout'];
        if ( $category_layout == 'slider' ) {
            wp_enqueue_style('slick');
            wp_enqueue_style('slick-theme');
            wp_enqueue_script('slick');
        }
        $category_style = $settings['woolisting_category_style'];
        $woolisting_slider_slides_to_show = $settings['woolisting_slider_slides_to_show'];
        $woolisting_slider_slides_to_show_tablet   =  $settings['woolisting_slider_slides_to_show_tablet'];
        $woolisting_slider_slides_to_show_mobile  = $settings['woolisting_slider_slides_to_show_mobile'];

        // Grid columns
        if ($settings['woolist_grid_columns'] == "1") {
            $column = "elementor-grid-1";
        } elseif ($settings['woolist_grid_columns'] == "2") {
            $column = "elementor-grid-2";
        } elseif ($settings['woolist_grid_columns'] == "3") {
            $column = "elementor-grid-3";
        } elseif ($settings['woolist_grid_columns'] == "4") {
            $column = "elementor-grid-4";
        } elseif ($settings['woolist_grid_columns'] == "5") {
            $column = "elementor-grid-5";
        } elseif ($settings['woolist_grid_columns'] == "6") {
            $column = "elementor-grid-6";
        }

         // Category per page.
        if (isset($settings['woolisting_number_of_category']) && !empty($settings['woolisting_number_of_category'])) {
            $args['number'] = $settings['woolisting_number_of_category'];
        } else {
            $args['number'] = -1;
        }

        $args = array(
            'taxonomy' => 'product_cat',
            'parent'   => 0,
            'number'   => $settings['woolisting_number_of_category'],
            'orderby'  => $settings['woolisting_category_orderby'],
            'order'    => $settings['woolisting_category_order'],
        );

        $categories = get_categories($args);

        if ($categories) :
            $this->add_render_attribute(
                'woolisting_slider_setting', [
                    'id'                                => 'woolisting-slider-' . $this->get_id(),
                    'data-woolisting-slides-to-show'          => $woolisting_slider_slides_to_show,
                    'data-woolisting-slides-to-show-tablet'   => $woolisting_slider_slides_to_show_tablet,
                    'data-woolisting-slides-to-show-mobile'   => $woolisting_slider_slides_to_show_mobile,
                    'data-woolisting-slider-autoplay'         => $settings['woolisting_slider_autoplay'],
                    'data-woolisting-slider-autoplay-speed'   => $settings['woolisting_slider_speed'],
                    'data-woolisting-slider-dots'             => $settings['woolisting_slider_dots'],
                    'data-woolisting-slider-arrows'           => $settings['woolisting_slider_arrows'],
                    'data-woolisting-slider-to-scroll'        => $settings['woolisting_slider_slides_to_scroll'],
                    'data-woolisting-slider-infinite'         => $settings['woolisting_slider_infinite'],
                    'data-woolisting-slider-pause-on-hover'   => $settings['woolisting_slider_pause_on_hover'], 
                    'data-woolisting-prev-arrow'              => Sliders_icons_managers1::render_icon($settings['woolisting_prev_icon'], [ 'aria-hidden' => 'true' ]),
                    'data-woolisting-next-arrow'              => Sliders_icons_managers1::render_icon($settings['woolisting_next_icon'], [ 'aria-hidden' => 'true' ]),
                ]
            );

        $gridClass = "";
        $sliderClass = "";
        $getAttributes = "";
        if($settings['woolisting_category_layout'] == "slider") {
            $sliderClass = "woolisting-category-slider";
            $getAttributes = $this->get_render_attribute_string('woolisting_slider_setting');
        }

        if($settings['woolisting_category_layout'] == "grid") {
            $gridClass = "grid-container product-grid-gap elementor-grid " .' ' . $column.' ';
        }
        ?>
        <div class="woolist-container woolist-category-slider<?php echo ' ' . $sliderClass. ' ' .$gridClass . ' ' . $category_style . ' ' . $category_layout; ?> product-grid-gap elementor-grid" <?php echo $getAttributes; ?>>
        <?php
        if ($categories) {
            foreach ($categories as $category) {
                switch ($settings['woolisting_category_style']) {
                        case 'woolisting-category-style-1':
                            include WOOLISTING_PATH . 'include/product-category-design/design-1.php'; // Design 1.
                            break;
                        case 'woolisting-category-style-2':
                            include WOOLISTING_PATH . 'include/product-category-design/design-2.php'; // Design 2.
                            break;
                        case 'woolisting-category-style-3':
                            include WOOLISTING_PATH . 'include/product-category-design/design-3.php'; // Design 3.
                            break;
                        case 'woolisting-category-style-4':
                            include WOOLISTING_PATH . 'include/product-category-design/design-4.php'; // Design 4.
                            break;
                        default:
                        include WOOLISTING_PATH . 'include/product-category-design/design-1.php'; // Default Design 1.
                        break;      
                    }
                }
            }    
            else {
                echo esc_html_e( 'Sorry, no Category were found.', 'woolisting' );
            }
        ?>
        </div>
        <?php
        endif;
    }
}
Plugin::instance()->widgets_manager->register(new WOOLISTING_CATEGORY());