<?php
namespace Elementor;
use Elementor\Icons_Manager;

// If this file is called directly, abort.
if (!defined('ABSPATH')) {
    exit;   
}

class WOOLISTING_Widget extends Widget_Base {

    // Function for get the slug of the element name.
    public function get_name() {
        return 'woolisting-product';
    }
    // Function for get the name of the element.
    public function get_title() {
        return esc_html__('WooListing Product', 'woolisting');
    }

    // Function for get the icon of the element.
    public function get_icon() {
        return  'eicon-posts-grid';
    }

    // Function for include element into the category.
    public function get_categories() {
        return ['woolisting'];
    }

    // Adding the controls fields for Woolisting Element.
    public function register_controls() {

        // Start Woolisting Product layout Section.
        $this->start_controls_section(
            'woolist_general_section', array(
                'label'                 => esc_html__('Product Layout', 'woolisting'),
            )
        );

        // Layout Style.
        $this->add_control(
            'woolisting_product_layout', [
            'label'                 => __('Layout Style', 'woolisting'),
            'type'                  => Controls_Manager::SELECT,
            'options'               => [
                'grid'              => esc_html__('Grid', 'woolisting'),
                'slider'            => esc_html__('Slider', 'woolisting'),
            ],
            'default'               => 'grid',
            ]
        );

        // Woolisting Design.
        $this->add_control(
            'woolisting_product_style', [
                'label'                 => esc_html__('Layout', 'woolisting'),
                'type'                  => Controls_Manager::SELECT,
                'options'               => [
                    'woolist-design-1'   => esc_html__('Design 1', 'woolisting'),
                    'woolist-design-2'   => esc_html__('Design 2', 'woolisting'),
                    'woolist-design-3'   => esc_html__('Design 3', 'woolisting'),
                    'woolist-design-4'   => esc_html__('Design 4', 'woolisting'),
                ],
                'default'               => 'woolist-design-1',
            ]
        );

        // Grid Columns.
        $this->add_responsive_control(
            'woolist_number_of_product', [
                'label'             => esc_html__('Grid Columns', 'woolisting'),
                'type'              => Controls_Manager::SELECT,
                'default'           => esc_html__('default', 'woolisting'),
                'options'           => [
                    'default'       => 'Default',
                    '1'             => '1',
                    '2'             => '2',
                    '3'             => '3',
                    '4'             => '4',
                    '5'             => '5',
                    '6'             => '6',
                ],  
                'prefix_class'      => 'elementor-grid%s-',
                'frontend_available'=> true,
                'selectors'         => [
                    '.elementor-msie {{WRAPPER}}  .woolist-container' => 'width: calc(100% / {{SIZE}} )',
                ],
                'condition'         => ['woolisting_product_layout' => 'grid'],
            ]
        );

        $this->end_controls_section();
        //End Woolisting Product layout Section.

        $this->start_controls_section(
            'woolist_item_section', array(
                'label'             => esc_html__('Item', 'woolisting'),
            )
        );

        //No.of Product.
        $this->add_control(
            'woolist_number_of_show_product', [
                'label'             => esc_html__('Display No. of Product', 'woolisting'),
                'type'              => Controls_Manager::NUMBER,
                'min'               => 1,
                'max'               => 100,
                'step'              => 1,
                'default'           => 5,
            ]
        );

        // Show Image.
        $this->add_control(
            'woolist_show_image', [
                'label'             => esc_html__('Show Image', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', 'woolisting'),
                'label_off'         => esc_html__('Hide', 'woolisting'),
                'default'           => 'yes',
                'separator'         => 'before',
                'condition'         => [
                    'woolisting_product_style' => ['woolist-design-1', 'woolist-design-2','woolist-design-3','woolist-design-4'],
                    ]
            ],
        );

        // Show Name.
        $this->add_control(
            'woolist_show_name', [
                'label'             => esc_html__('Show Product Name', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', 'woolisting'),
                'label_off'         => esc_html__('Hide', 'woolisting'),
                'default'           => 'yes',
                ]
        );

        // Show Description.
        $this->add_control(
            'woolist_show_description', [
                'label'             => esc_html__('Show Description', 'woolisting'),
                'type'              => Controls_Manager::SELECT,
                'options'           => [
                    'none'          => esc_html__('None', 'woolisting'),
                    'short'         => esc_html__('Short', 'woolisting'),
                    'full'          => esc_html__('Full', 'woolisting'),
                ],
                'default'           => 'short',
            ]
        );

        // Content Length.
        $this->add_control(
            'woolist_description_length', [
                'label'             => esc_html__('Description Length', 'woolisting'),
                'type'              => Controls_Manager::NUMBER,
                'default'           => apply_filters('woolist_content_length', 15),
                'condition'         => [
                    'woolist_show_description'  => ['full' , 'short'],
                ],
            ]
        );

        // Show Regular price.
        $this->add_control(
            'woolist_show_regular_price', [
                'label'             => esc_html__('Show Regular Price', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
            'label_on'          => esc_html__('Show', 'woolisting'),
            'label_off'         => esc_html__('Hide', 'woolisting'),
            'default'           => 'yes',
            ]
        );

        // Show Sale Price.
        $this->add_control(
            'woolist_show_sale_price', [
            'label'             => esc_html__('Show Sale Price', 'woolisting'),
            'type'              => Controls_Manager::SWITCHER,
            'label_on'          => esc_html__('Show', 'woolisting'),
            'label_off'         => esc_html__('Hide', 'woolisting'),
            'default'           => 'yes',
            ]
        );

        // Show Customer Rating.
        $this->add_control(
            'woolist_show_customer_rating', [
                'label'             => esc_html__('Show Customer Rating', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', 'woolisting'),
                'label_off'         => esc_html__('Hide', 'woolisting'),
                'default'           => 'yes',
            ]
        );

        // WooCommerce Sale Label.
        $this->add_control(
            'woolist_sale', [
                'label'             => esc_html__('Sale', 'woolisting'),
                'type'              => Controls_Manager::HEADING,
                
            ]   
        );

        // WooCommerce Show Sale Label.
        $this->add_control(
            'woolist_show_label', [
                'label'             => esc_html__('Sale Label', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', 'woolisting'),
                'label_off'         => esc_html__('Hide', 'woolisting'),
                'default'           => 'yes',
                'separator'         => 'before',
            ]
        );

        // WooCommerce Sale Label Text.
        $this->add_control(
            'woolist_sale_label', [
                'label'             => esc_html__('Sale Label Text', 'woolisting'),
                'type'              => Controls_Manager::TEXT,
                'default'           => esc_html__('Sale', 'woolisting'),
                'condition'         => ['woolist_show_label' => 'yes'],
            ]
        );

        // WooCommerce Button.
        $this->add_control(
            'woolist_button', [
                'label'             => esc_html__('Button', 'woolisting'),
                'type'              => Controls_Manager::HEADING,
                
            ]
        );

        // Show Cart Button.
        $this->add_control(
            'woolist_show_cart_button', [
                'label'             => esc_html__('Show Cart Button', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', 'woolisting'),
                'label_off'         => esc_html__('Hide', 'woolisting'),
                'default'           => 'yes',
                'separator'         => 'before',
            ]
        );

        // Show Custom Button.
        $this->add_control(
            'woolist_show_custom_button', [
                'label'             => esc_html__('Show Custom Button', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', 'woolisting'),
                'label_off'         => esc_html__('Hide', 'woolisting'),
                'default'           => 'yes',
                'condition'         => ['woolist_show_cart_button' => 'yes'],
            ]
        );

        // Simple Product Text.
        $this->add_control(
            'woolist_simple_button_text', [
                'label'             => esc_html__('Simple Product Text', 'woolisting'),
                'type'              => Controls_Manager::TEXT,
                'default'           => esc_html__('Order Now', 'woolisting'),
                'condition'         => ['woolist_show_custom_button' => 'yes'],
            ]   
        );

        //Grouped Product Text.
        $this->add_control(
            'woolist_grouped_button_text', [
                'label'             => esc_html__('Grouped Product Text', 'woolisting'),
                'type'              => Controls_Manager::TEXT,
                'default'           => esc_html__('Order Now', 'woolisting'),
                'condition'         => ['woolist_show_custom_button' => 'yes'],
            ]
        );

        //External/Affiliate Button Text.
        $this->add_control(
            'woolist_external_button_text', [
                'label'             => esc_html__('External/Affiliate Button Text', 'woolisting'),
                'type'              => Controls_Manager::TEXT,
                'default'           => esc_html__('Order Now', 'woolisting'),
                'condition'         => ['woolist_show_custom_button' => 'yes'],
            ]
        );

        //Variable Product Text.
        $this->add_control(
            'woolist_variable_button_text', [
                'label'             => esc_html__('Variable Product Text', 'woolisting'),
                'type'              => Controls_Manager::TEXT,
                'default'           => esc_html__('Order Now', 'woolisting'),
                'condition'         => ['woolist_show_custom_button' => 'yes'],
            ]
        );

        $this->end_controls_section();
        //End Woolisting List Item layout Section.

        //Start Products Query Section.
        $this->start_controls_section(
            'woolist_product_query_section', array(
                'label'             => esc_html__('Products Query', 'woolisting'),
            )
        );

        //Product Source.
        $this->add_control(
            'woolisting_product_source', [
                'label'         => esc_html__('Product Source', 'woolisting'),
                'label_block'   => true,
                'type'          => Controls_Manager::SELECT,
                'default'       =>  'select',
                'multiple'      => true,
                'options'       => [
                'select'                => esc_html__('Select', 'woolisting' ),
                'recent-products'       => esc_html__('Recent Products', 'woolisting' ),
                'featured-products'     => esc_html__('Featured Products', 'woolisting' ),
                'best-selling-products' => esc_html__('Best Selling Products', 'woolisting' ),
                'sale-products'         => esc_html__('Sale Products', 'woolisting' ),
                'top-products'          => esc_html__('Top Rated Products', 'woolisting' ),
                ],
            ]
        );

        //Include Categories.
        $this->add_control(
            'woolist_product_categories', [
                'label'             => esc_html__('Product Categories', 'woolisting'),
                'type'              => Controls_Manager::SELECT2,
                'label_block'       => true,
                'multiple'          => true,
                'description'       => esc_html__('Select the categories you want to show', 'woolisting'),
                'separator'         => 'before',
                'options'           => woolist_product_categories(),
            ]
        );

        //Exclude Categories.
        $this->add_control(
            'woolist_exclude_product_categories', [
                'label'             => esc_html__('Exclude Above Categories', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Yes', 'woolisting'),
                'label_off'         => esc_html__('No', 'woolisting'),
                'default'           => 'yes',
            ]
        );

        //Include Tags.
        $this->add_control(
            'woolist_product_tags', [
                'label'             => esc_html__('Product Tags', 'woolisting'),
                'type'              => Controls_Manager::SELECT2,
                'label_block'       => true,
                'multiple'          => true,
                'description'       => esc_html__('Select the tags you want to show', 'woolisting'),
                'options'           => woolist_product_tags(),
                'separator'         => 'before',
            ]
        );

        //Exclude Tags.
        $this->add_control(
            'woolist_exclude_product_tags', [
                'label'             => esc_html__('Exclude Above Tags', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Yes', 'woolisting'),
                'label_off'         => esc_html__('No', 'woolisting'),
                'default'           => 'yes',
            ]
        );

        // Post Order By.
        $this->add_control(
            'woolist_category_orderby', [
                'label'             => esc_html__('Order By', 'woolisting'),
                'type'              => Controls_Manager::SELECT,
                'default'           => 'title',
                'options'           => [
                'date'              => esc_html__('Date', 'woolisting'),
                'title'             => esc_html__('Name', 'woolisting'),
                'rand'              => esc_html__('Random', 'woolisting'),
                'ID'                => esc_html__('ID', 'woolisting'),
                ],
            ]
        );

        // Post Sort By.
        $this->add_control(
            'woolist_category_order', [
                'label'             => esc_html__('Sort By', 'woolisting'),
                'type'              => Controls_Manager::SELECT,
                'default'           => 'desc',
                'options'           => [
                    'asc'           => esc_html__('ASC', 'woolisting'),
                    'desc'          => esc_html__('DESC', 'woolisting'),
                ],
            ]
        );

        $this->end_controls_section();
        // End Products Query Section.

        // Grid Configuration.
        $this->start_controls_section(
            'woolisting_grid_options', [
                'label'             => esc_html__('Grid Configuration', 'woolisting'),
                'tab'               => Controls_Manager::TAB_CONTENT,
                'condition'         => ['woolisting_product_layout' => 'grid'],
            ]
        );

        // Grid Column Gap.
        $this->add_responsive_control(
            'woolisting_product_column_gap', [
                'label'             => esc_html__('Grid Column Gap', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'default'           => [
                    'size'          => 30,
                ],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available'  => true,
                'selectors'         => [
                    '{{WRAPPER}} .product-grid-gap' => 'grid-column-gap: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Grid Row Gap.
        $this->add_responsive_control(
            'woolisting_product_row_gap', [
                'label'             => esc_html__('Grid Row Gap', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'default'           => [
                    'size'          => 30,
                ],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available'  => true,
                'selectors'         => [
                    '{{WRAPPER}} .product-grid-gap' => 'grid-row-gap: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        // End Grid Configuration.

        // Start Slider Configuration.
        $this->start_controls_section(
            'woolisting_slider_options', [
                'label'             => esc_html__('Slider Configuration', 'woolisting'),
                'tab'               => Controls_Manager::TAB_CONTENT,
                'condition'         => ['woolisting_product_layout' => 'slider'],
            ]
        );

        // Slide to show.
        $this->add_responsive_control(
            'woolisting_slider_slides_to_show', [
                'label'             => esc_html__('Slides To Show', 'woolisting'),
                'type'              => Controls_Manager::NUMBER,
                'default'           => 3,
                'devices'           => ['desktop', 'tablet', 'mobile'],
                'desktop_default'   => 3,
                'tablet_default'    => 2,
                'mobile_default'    => 1,
                'frontend_available'  => true,
            ]
        );

        // Slide to Scroll.
        $this->add_control(
            'woolisting_slider_slides_to_scroll', [
                'label'             => esc_html__('Slides To Scroll', 'woolisting'),
                'type'              => Controls_Manager::NUMBER,
                'min'               => 1,
                'max'               => 6,
                'step'              => 1,
                'default'           => 1,
            ]
        );

        // Slider Infinite.
        $this->add_control(
            'woolisting_slider_infinite', [
                'label'             => esc_html__('Show Infinite', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', 'woolisting'),
                'label_off'         => esc_html__('Hide', 'woolisting'),
                'return_value'      => 'yes',
                'default'           => 'no',
            ]
        );

        // Pause On Hover.
        $this->add_control(
            'woolisting_slider_pause_on_hover', [
                'label'            => esc_html__('Pause On Hover', 'woolisting'),
                'type'             => Controls_Manager::SWITCHER,
                'label_on'         => esc_html__('Show', 'woolisting'),
                'label_off'        => esc_html__('Hide', 'woolisting'),
                'return_value'     => 'yes',
                'default'          => 'no',
            ]
        );

        // Slider Autoplay.
        $this->add_control(
            'woolisting_slider_autoplay', [
                'label'             => esc_html__('Autoplay', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'default'           => 'yes',
                'label_off'         => esc_html__('Off', 'woolisting'),
                'label_on'          => esc_html__('On', 'woolisting'),
            ]
        );

        // Autoplay Speed.
        $this->add_control(
            'woolisting_slider_speed', [
                'label'             => esc_html__('Autoplay Speed', 'woolisting'),
                'type'              => Controls_Manager::NUMBER,
                'min'               => 1000,
                'max'               => 10000,
                'step'              => 1000,
                'default'           => 5000,
                'description'       => esc_html__('Speed is in milliseconds (1000ms = 1s).', 'woolisting')
            ]
        );

        // Show Dots.
        $this->add_control(
            'woolisting_slider_dots', [
                'label'             => esc_html__('Show Dots', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', 'woolisting'),
                'label_off'         => esc_html__('Hide', 'woolisting'),
                'return_value'      => 'yes',
                'default'           => 'no',
            ]
        );

        // Show Arrows.
        $this->add_control(
            'woolisting_slider_arrows', [
                'label'            => esc_html__('Show Arrows', 'woolisting'),
                'type'             => Controls_Manager::SWITCHER,
                'label_on'         => esc_html__('Show', 'woolisting'),
                'label_off'        => esc_html__('Hide', 'woolisting'),
                'return_value'     => 'yes',
                'default'          => 'yes',
            ]
        );

        // Previous Icon.
        $this->add_control(
            'woolisting_prev_icon', [
                'label'            => esc_html__('Previous icon', 'woolisting'),
                'type'             => Controls_Manager::ICONS,
                'default'          => [
                    'value'        => 'fas fa-chevron-left',
                    'library'      => 'solid',
                ],
            ]
        );

        // Next Icon.
        $this->add_control(
            'woolisting_next_icon', [
                'label'            => esc_html__('Next icon', 'woolisting'),
                'type'             => Controls_Manager::ICONS,
                'default'          => [
                'value'            => 'fas fa-chevron-right',
                'library'          => 'solid',
                ]
            ]
        );

        // Space Between.
        $this->add_responsive_control(
            'woolisting_slider_gap', [
                'label'             => esc_html__('Column Gap', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'default'           => [
                    'size'          => 10,
                ],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available'  => true,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-slide > div' => 'margin-left: {{SIZE}}{{UNIT}}; margin-right: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section(); 
        // End Slider Configuration.

        // Style Section Start.
        // Start Item Style.
        $this->start_controls_section(
            'woolist_item_style', [
                'label'             => esc_html__('Item', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        // Item Background Color.
        $this->add_control( 
            'woolist_item_bg_color', [
                'label'             => esc_html__('Item Background Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-product-1,
                    {{WRAPPER}} .woolist-product-2,
                    {{WRAPPER}} .woolist-product-3,
                    {{WRAPPER}} .woolist-product-4' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        // Item Border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woolist_item_border',
                'selector'          => '{{WRAPPER}} .woolist-product-1,
                {{WRAPPER}} .woolist-product-2,
                {{WRAPPER}} .woolist-product-3,
                {{WRAPPER}} .woolist-product-4',
            ]
        );

        // Item Box Shadow.
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(), [
                'name'              => 'woolist_box_shadow',
                'selector'          => '{{WRAPPER}} .woolist-product-1,
                {{WRAPPER}} .woolist-product-2,
                {{WRAPPER}} .woolist-product-3,
                {{WRAPPER}} .woolist-product-4',
            ]
        );

        // Item Padding.
        $this->add_responsive_control(
            'woolist_item_padding', [
                'label'             => esc_html__('Item Padding', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-product-1,
                    {{WRAPPER}} .woolist-product-2,
                    {{WRAPPER}} .woolist-product-3,
                    {{WRAPPER}} .woolist-product-4' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        // Item Content Padding.
        $this->add_responsive_control(
            'woolist_content_padding', [
                'label'             => esc_html__('Content Padding', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-content-1,
                    {{WRAPPER}} .woolist-product-2,
                    {{WRAPPER}} .woolist-product-3,
                    {{WRAPPER}} .woolist-product-4' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        // End Item Style.

        // Start Image Style.
        $this->start_controls_section(
            'woolist_image_style', [
                'label'             => esc_html__('Image', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        // Content Alignment.
        $this->add_responsive_control(
            'woolist_image_align', [
                'label'             => esc_html__('Alignment', 'woolistiing'),
                'type'              => Controls_Manager::CHOOSE,
                'options'           => [
                    'left'          => [
                        'title'     => esc_html__('Left', 'woolistiing'),
                        'icon'      => 'eicon-text-align-left',
                    ],
                    'center'        => [
                        'title'     => esc_html__('Center', 'woolistiing'),
                        'icon'      => 'eicon-text-align-center',
                    ],
                    'right'         => [
                        'title'     => esc_html__('Right', 'woolistiing'),
                        'icon'      => 'eicon-text-align-right',
                    ],
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-product-1 .woolist-img,
                    {{WRAPPER}} .woolist-product-2 .woolist-img,
                    {{WRAPPER}} .woolist-product-3 .woolist-img' => 'justify-content: {{VALUE}};',
                ],
                'condition'         => [
                    'woolisting_product_style' => ['woolist-design-1', 'woolist-design-2', 'woolist-design-3'],
                ]
            ]
        );

        // Image Border Radius.
        $this->add_responsive_control(
            'woolist_image_border_radius', [
                'label'             => esc_html__('Image Border Radius', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-product-img-1 img,
                    {{WRAPPER}} .woolist-product-img-2 a,
                    {{WRAPPER}} .woolist-product-img-3 a' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolisting_product_style' => ['woolist-design-1', 'woolist-design-2'],
                ]
            ]
        );

        // Image Border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woolist_image_border',
                'selector'          => '{{WRAPPER}} .woolist-product-img-1 img,
                {{WRAPPER}} .woolist-product-img-2 a',
                'condition'         => [
                    'woolisting_product_style' => ['woolist-design-1', 'woolist-design-2'],
                ]
            ]
        );

        // Image Padding.
        $this->add_responsive_control(
            'woolist_image_padding', [
                'label'             => esc_html__('Padding', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-product-img-1 img,
                    {{WRAPPER}} .woolist-product-img-2 img,
                    {{WRAPPER}} .woolist-product-img-3 img,
                    {{WRAPPER}} .woolist-product-img-4 img' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]   
        );

        // Image Width.
        $this->add_responsive_control(
            'woolist_image_width', [
                'label'             => esc_html__('Image Width', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => [ 'px', '%' ],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 500,
                    ],
                    '%'             => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-product-img-1 img,
                    {{WRAPPER}} .woolist-product-img-2 img,
                    {{WRAPPER}} .woolisting-product-slider .woolist-product-img-2 a,
                    {{WRAPPER}} .woolisting-product-slider .woolist-product-img-3 img,
                    {{WRAPPER}} .woolist-product-img-3 img,
                    {{WRAPPER}} .woolist-product-img-4 img'  => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Image Width.
        $this->add_responsive_control(
            'woolist_image_height', [
                'label'             => esc_html__('Image Height', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => [ 'px', '%' ],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 500,
                    ],
                    '%'             => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-product-img-1 img,
                    {{WRAPPER}} .woolist-product-img-2 img,
                    {{WRAPPER}} .woolist-product-img-3 img,
                    {{WRAPPER}} .woolist-product-img-4 img' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Image Overlay.
        $this->add_control(
            'acle_overlay_color', [
                'label'            => esc_html__('Image Overlay', 'woolisting'),
                'type'             => Controls_Manager::COLOR,
                'selectors'        => [
                    '{{WRAPPER}} .woolist-content-4' => 'background-color: {{VALUE}};',
                ],
                'condition'         => [
                    'woolisting_product_style' => 'woolist-design-4',
                ]
            ]
        );

        $this->end_controls_section();
        // End Image Style.

        // Start Title Style.
        $this->start_controls_section(
            'woolist_title_style', [
                'label'             => esc_html__('Title', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_responsive_control(
            'woolist_title_align', [
                'label'             => esc_html__('Alignment', 'woolistiing'),
                'type'              => Controls_Manager::CHOOSE,
                'options'           => [
                    'left'          => [
                        'title'     => esc_html__('Left', 'woolistiing'),
                        'icon'      => 'eicon-text-align-left',
                    ],
                    'center'        => [
                        'title'     => esc_html__('Center', 'woolistiing'),
                        'icon'      => 'eicon-text-align-center',
                    ],
                    'right'         => [
                        'title'     => esc_html__('Right', 'woolistiing'),
                        'icon'      => 'eicon-text-align-right',
                    ],
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-title' => 'text-align: {{VALUE}};',
                ],
                'condition'         => [
                    'woolisting_product_style' => 'woolist-design-1', 
                ]
            ]
        );

        // Title Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'              => 'woolist_title_typography',
                'selector'          => '{{WRAPPER}} .woolist-title a, 
                {{WRAPPER}} .woolist-title-1 a,
                {{WRAPPER}} .woolist-title-3 a,
                {{WRAPPER}} .woolist-title-4 a',
            ],
            
        );        

        // Title Color.
        $this->add_control( 
            'woolist_title_color', [
                'label'             => esc_html__('Title Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-title a,
                    {{WRAPPER}} .woolist-title-1 a,
                    {{WRAPPER}} .woolist-title-3 a,
                    {{WRAPPER}} .woolist-title-4 a' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Title Spacing.
        $this->add_responsive_control(
            'woolist_title_spacing', [
                'label'             => esc_html__('Title Spacing', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-container .woolist-title' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );
        $this->end_controls_section();
        // End Title Style.

        // Start Description Style.
        $this->start_controls_section(
            'woolist_description_style', [
                'label'             => esc_html__('Description', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        // Description Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'              => 'woolist_description_typography',
                'selector'          => '{{WRAPPER}} .woolist-show-description p,
                {{WRAPPER}} .woolist-show-description-4 p',
            ]
        );

        // Description Text Color.
        $this->add_control( 
            'woolist_description_color', [
                'label'             => esc_html__('Description Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-show-description p,
                    {{WRAPPER}} .woolist-show-description-4 p' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Description Spacing.
        $this->add_responsive_control(
            'woolist_description_spacing', [
                'label'             => esc_html__('Description Spacing', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 50,
                    ],
                ],
                'frontend_available' => true,
                'selectors'          => [
                    '{{WRAPPER}} .woolist-show-description p' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolisting_product_style' => ['woolist-design-1', 'woolist-design-2', 'woolist-design-3'],
                ]
            ]
        );

        $this->end_controls_section();
        // End Description Style.

        // Start Price Style.
        $this->start_controls_section(
            'woolist_price_style', [
                'label'             => esc_html__('Price', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        // Regular Price Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'              => 'woolist_regular_price_typography',
                'label'             => esc_html__('Regular Price Typography', 'woolisting'),
                'selector'          => '{{WRAPPER}} .woolist-regular-price,
                {{WRAPPER}} .woolist-regular-price-4',
            ]
        );

        // Regular Price Color.
        $this->add_control( 
            'woolist_regular_color', [
                'label'             => esc_html__('Regular Price Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-regular-price,
                    {{WRAPPER}} .woolist-regular-price-4' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Show Line Through.
        $this->add_control(
            'woolist_show_line_through', [
                'label'             => esc_html__('Show Line Through', 'woolisting'),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', 'woolisting'),
                'label_off'         => esc_html__('Hide', 'woolisting'),
                'default'           => 'yes',
                
            ],
        );

        // Regular Price line through width
        $this->add_responsive_control(
            'woolist_line_width', [
                'label' 			=> esc_html__('Price Line Through Width', 'woolisting'),
                'type' 				=> Controls_Manager::SLIDER,
                'size_units' 		=> ['px'],
                'range'				=> [
                    'px' 			=> [
                        'min' 		=> 0,
                        'max' 		=> 10,
                        'step' 		=> 1,
                    ],
                ],
                'default' 			=> [
                    'unit' 			=> 'px',
                    'size' 			=> 1,
                ],
                'selectors' 		=> [
                    '{{WRAPPER}} .woolist-line-price' => 'text-decoration-thickness: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolist_show_line_through' => 'yes',
                ]
            ]
        );

        // Regular Price line through.
        $this->add_control(
            'woolist_price_line_color', [
                'label'         	=> esc_html__('Price Line Through', 'woolisting'),
                'type'          	=> Controls_Manager::COLOR,
                'selectors'     	=> [
                    '{{WRAPPER}} .woolist-line-price' => 'text-decoration-color: {{VALUE}};',
                ],
            ]
        );

        // Sale Price Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'              => 'woolist_sale_price_typography',
                'label'             => esc_html__('Sale Price Typography', 'woolisting'),
                'selector'          => '{{WRAPPER}} .woolist-sale-price,
                {{WRAPPER}} .woolist-sale-price-4',
            ]
        );

        // Sale Price Color.
        $this->add_control( 
            'woolist_sale_color', [
                'label'             => esc_html__('Sale Price Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-sale-price,
                    {{WRAPPER}} .woolist-sale-price-4' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Price Spacing.
        $this->add_responsive_control(
            'woolist_price_spacing', [
                'label'             => esc_html__('Price Spacing', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 80,
                    ],
                ],
                'default'           => [
                    'unit'          => 'px',
                    'size'          => 5,
                ],
                'frontend_available' => true,
                'selectors'          => [
                    '{{WRAPPER}} .woolist-sale-price,
                    {{WRAPPER}} .woolist-sale-price-4' => 'margin-left: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        // End Price Style.

        // Start Button Style.
        $this->start_controls_section(
            'woolist_button_style', [
                'label'             => esc_html__('Button', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        // Button Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'              => 'woolist_button_typography',
                'selector'          => '{{WRAPPER}} .woolist-button a,
                {{WRAPPER}}     .woolist-button-2 a,
                {{WRAPPER}}     .woolist-button-3 a,
                {{WRAPPER}}     .woolist-button-4 a',
            ]
        );

        // Button Spacing.
        $this->add_responsive_control(
            'woolist_button_spacing', [
                'label'             => esc_html__('Button Spacing', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 50,
                    ],
                ],
                'frontend_available' => true,
                'selectors'          => [
                    '{{WRAPPER}} .woolist-button-wrapper,
                    {{WRAPPER}}  .woolist-button-wrapper-2,
                    {{WRAPPER}}  .woolist-button-3,
                    {{WRAPPER}}  .woolist-button-4' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );
        
        // Start Button Tab.
        $this->start_controls_tabs('woolist_button_color_tabs');

        // Start Normal Button Tab.
        $this->start_controls_tab('woolist_normal_button', [
            'label'                 => esc_html__('Normal', 'woolisting'),
            ]
        );

        // Button Text Color.
        $this->add_control(
            'woolist_button_color', [
                'label'             => esc_html__('Text Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}}    .woolist-button a,
                    {{WRAPPER}}     .woolist-button-2 a,
                    {{WRAPPER}}     .woolist-button-3 a,
                    {{WRAPPER}}     .woolist-button-4 a' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Button Background Color.
        $this->add_control(
            'woolist_button_bg_color', [
                'label'             => esc_html__('Button Background Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}}    .woolist-button a,
                    {{WRAPPER}}     .woolist-button-2 a,
                    {{WRAPPER}}     .woolist-button-3 a,
                    {{WRAPPER}}     .woolist-button-4 a' => 'background-color: {{VALUE}};',
                ],
                'separator'         => 'after',
            ]
        );

        $this->end_controls_tab();
        // End Normal Button Tab.

        // Start Hover Button Tab.
        $this->start_controls_tab('woolist_hover_button', [
            'label'                 => esc_html__('Hover', 'woolisting'),
            ]
        );

        // Button Text Color.
        $this->add_control(
            'woolist_button_color_hover', [
                'label'             => esc_html__('Text Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}}    .woolist-button a:hover,
                    {{WRAPPER}}     .woolist-button-2 a:hover,
                    {{WRAPPER}}     .woolist-button-3 a:hover,
                    {{WRAPPER}}     .woolist-button-4 a:hover' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Button Background Color.
        $this->add_control(
            'woolist_button_bg_color_hover', [
                'label'             => esc_html__('Button Background Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}}   .woolist-button a:hover,
                    {{WRAPPER}}    .woolist-button-2 a:hover,
                    {{WRAPPER}}    .woolist-button-3 a:hover,
                    {{WRAPPER}}    .woolist-button-4 a:hover' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        // Button border.
        $this->add_control(
            'woolist_border_color_hover', [
                'label'             => esc_html__('Border Hover Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}}    .woolist-button a:hover,
                    {{WRAPPER}}     .woolist-button-2 a:hover,
                    {{WRAPPER}}     .woolist-button-3 a:hover,
                    {{WRAPPER}}     .woolist-button-4 a:hover' => 'border-color: {{VALUE}};',
                ],
                'separator'         => 'after',
            ]
        );

        $this->end_controls_tab();
        // End Hover Button Tab.

        $this->end_controls_tabs();
        // End Button Tab.

        // Button border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woolist_button_border',
                'selector'          => '{{WRAPPER}} .woolist-button a,
                {{WRAPPER}} .woolist-button-2 a,
                {{WRAPPER}} .woolist-button-3 a,
                {{WRAPPER}} .woolist-button-4 a',
            ]
        );

        // Button Border Radius.
        $this->add_responsive_control(
            'woolist_border_radius', [
                'label'             => esc_html__('Border Radius', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}}   .woolist-button a,
                    {{WRAPPER}}    .woolist-button-2 a,
                    {{WRAPPER}}    .woolist-button-3 a,
                    {{WRAPPER}}    .woolist-button-4 a' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        // Button Border Radius.
        $this->add_responsive_control(
            'woolist_button_padding', [
                'label'             => esc_html__('Button Padding', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-button a,
                    {{WRAPPER}} .woolist-button-2 a,
                    {{WRAPPER}} .woolist-button-3 a,
                    {{WRAPPER}} .woolist-button-4 a' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
        $this->end_controls_section();
        // End Button Style.

        // Start Rating Style.
        $this->start_controls_section(
            'woolist_rating_style', [
                'label'             => esc_html__('Rating', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        // Rating Spacing.
        $this->add_responsive_control(
            'woolist_rating_spacing', [
                'label'             => esc_html__('Rating Spacing', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-customer-rate-1 i,
                    {{WRAPPER}} .woolist-customer-rate-2 i,
                    {{WRAPPER}} .woolist-customer-rate-4 i' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Rating Size.
        $this->add_control(
            'woolist_rating_size', [
                'label'             => esc_html__('Rating Size', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => [ 'px', '%' ],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                        'step'      => 1,
                    ],
                ],
                'default'           => [
                    'size'          => 14,
                    'unit'          => 'px',
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-customer-rate-1 i,
                    {{WRAPPER}} .woolist-customer-rate-2 i,
                    {{WRAPPER}} .woolist-customer-rate-4 i' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Rating Fill Color.
        $this->add_control(
            'woolist_rating_fill_color', [
                'label'             => esc_html__('Rating Fill Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .star-fill' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Rating Color.
        $this->add_control(
            'woolist_rating_color', [
                'label'             => esc_html__('Rating Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}}  .star' => 'color: {{VALUE}};',
                ],
            ]
        );
        
        $this->end_controls_section();
        // End Rating Style.

        // Start Label Style.
        $this->start_controls_section(
            'woolist_label_style', [
                'label'             => esc_html__('Label', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        // Label Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'              => 'woolist_label_typography',
                'selector'          => '{{WRAPPER}} .woolist-ribbon .woolist-label-1,
                {{WRAPPER}} .woolist-label-2,
                {{WRAPPER}} .woolist-label-3 span,
                {{WRAPPER}} .woolist-label-4',
            ]
        );

        // Label Color.
        $this->add_control( 
            'woolist_label_color', [
                'label'             => esc_html__('Label Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-label-1,
                    {{WRAPPER}} .woolist-label-2,
                    {{WRAPPER}} .woolist-label-3,
                    {{WRAPPER}} .woolist-label-4' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Label Background Color.
        $this->add_control( 
            'woolist_label_bg_color', [
                'label'             => esc_html__('Label Background Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolist-label-1,
                    {{WRAPPER}} .woolist-label-2,   
                    {{WRAPPER}} .woolist-label-3::after,
                    {{WRAPPER}} .woolist-label-4' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        // Label Border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woolist_label_border',
                'selector'          => '{{WRAPPER}} .woolist-label-1,
                {{WRAPPER}} .woolist-label-2,
                {{WRAPPER}} .woolist-label-4',
                'condition'         => [
                    'woolisting_product_style' => ['woolist-design-1', 'woolist-design-2','woolist-design-4'],
                ]
            ]
        );

        // Label Border Radius.
        $this->add_responsive_control(
            'woolist_label_border_radius', [
                'label'             => esc_html__('Label Border Radius', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-label-1,
                    {{WRAPPER}} .woolist-label-2' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolisting_product_style' => ['woolist-design-1', 'woolist-design-2'],
                ]
            ]
        );

        // Label Padding.
        $this->add_responsive_control(
            'woolist_label_padding', [
                'label'             => esc_html__('Label Padding', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-label-1,
                    {{WRAPPER}} .woolist-label-2' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolisting_product_style!' => 'woolist-design-6',
                    'woolisting_product_style' => ['woolist-design-1', 'woolist-design-2'],
                ]
            ]
        );

        // Label Spacing
        $this->add_responsive_control(
            'woolist_label_spacing', [
                'label'             => esc_html__('Label Spacing', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => [ 'px', '%' ],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                        'step'      => 1,
                    ],
                ],
                'default'           => [
                    'size'          => 10,
                    'unit'          => 'px',
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolist-label-1,
                    {{WRAPPER}} .woolist-label-2' => 'margin-right: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woolisting_product_style' => 'woolist-design-1','woolist-design-2',
                ]
            ]
        );

        $this->end_controls_section();
        // End Label Style Control Section

        // Start Pagination Style.
        $this->start_controls_section(
            'woolisting_pagination_style', [
                'label'             => esc_html__('Pagination', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
                'condition'         => ['woolisting_product_layout' => 'slider'],
            ]
        );

        // Pagination Color.
        $this->add_control(
            'woolisting_pagination_color', [
                'label'             => esc_html__('Pagination Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'default'           => '#FD5056',
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-dots li button::before' => 'color: {{VALUE}} !important;',
                ],
            ]
        );

        // Active Pagination Color.
        $this->add_control(
            'woolisting_active_pagination_color', [
                'label'             => esc_html__('Active Pagination Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'default'           => '#FD5056',
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-dots li.slick-active button::before' => 'color: {{VALUE}} !important;',
                ],
            ]
        );

        // Pagination Size.
        $this->add_responsive_control(
            'woolisting_pagination_size', [
                'label'             => esc_html__('Pagination Size', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => ['px', '%'],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                        'step'      => 1,
                    ],
                ],
                'default'           => [
                    'size'          => 40,
                    'unit'          => 'px',
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-dots li button::before' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Active Pagination Size.
        $this->add_responsive_control(
            'woolisting_active_pagination_size', [
                'label'             => esc_html__('Active Pagination Size', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => ['px', '%'],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                        'step'      => 1,
                    ],
                ],
                'default'           => [
                    'size'          => 40,
                    'unit'          => 'px',
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-dots li.slick-active button::before' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Pagination Size.
        $this->add_responsive_control(
            'woolisting_pagination_square', [
                'label'             => esc_html__('Pagination Size', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => ['px', '%'],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                        'step'      => 1,
                    ],
                ],
                'default'           => [
                    'size'          => 14,
                    'unit'          => 'px',
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-dots li button::before,
                    {{WRAPPER}} .woolisting-product-slider .slick-dots li button::before' => 'width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        // End Pagination Style.

        // Start Navigation Style. 
        $this->start_controls_section(
            'woolisting_arrow_style', [
                'label'             => esc_html__('Arrows', 'woolisting'),
                'tab'               => Controls_Manager::TAB_STYLE,
                'condition'         => ['woolisting_product_layout' => 'slider'],
            ]
        );

        // Arrow Size.
        $this->add_responsive_control(
            'woolisting_arrow_size', [
                'label'             => esc_html__('Arrow Size', 'woolisting'),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => ['px', '%'],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                        'step'      => 1,
                    ],
                ],
                'default'           => [
                    'size'          => 18,
                    'unit'          => 'px',
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-prev i,
                    {{WRAPPER}} .woolisting-product-slider .slick-next i' => 'font-size: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .woolisting-product-slider .slick-prev svg,
                    {{WRAPPER}} .woolisting-product-slider .slick-next svg' => 'width:{{SIZE}}{{UNIT}}; height:{{SIZE}}{{UNIT}};',
                ],
            ]
        );

        // Start Item Tab.
        $this->start_controls_tabs('woolisting_arrow_tabs');

        // Start Normal Item Tab.
        $this->start_controls_tab('woolisting_normal_arrow', [
            'label'                => esc_html__('Normal', 'woolisting'),
            ]
        );

        // Arrow Color.
        $this->add_control(
            'woolisting_arrow_color', [
                'label'             => esc_html__('Arrow Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-prev i,
                    {{WRAPPER}} .woolisting-product-slider .slick-next i' => 'color: {{VALUE}} !important;',
                    '{{WRAPPER}} .woolisting-product-slider .slick-prev svg,
                    {{WRAPPER}} .woolisting-product-slider .slick-next svg' => 'fill:{{VALUE}} !important;',
                ],
            ]
        );

        // Arrow Background Color.
        $this->add_control(
            'woolisting_arrow_background_color', [
                'label'             => esc_html__('Arrow Background Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-prev i,
                    {{WRAPPER}} .woolisting-product-slider .slick-next i' => 'background-color: {{VALUE}} !important;',
                ],
            ]
        );

        $this->end_controls_tab();
        // End Normal Item Tab.

        // Start Hover Item Tab.
        $this->start_controls_tab('woolisting_hover_arrow', [
            'label'                => esc_html__('Hover', 'woolisting'),
            ]
        );

        // Arrow Color.
        $this->add_control(
            'woolisting_arrow_color_hover', [
                'label'             => esc_html__('Arrow Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-prev i:hover,
                    {{WRAPPER}} .woolisting-product-slider .slick-next i:hover' => 'color: {{VALUE}} !important;',
                    '{{WRAPPER}} .woolisting-product-slider .slick-prev svg:hover,
                    {{WRAPPER}} .woolisting-product-slider .slick-next svg:hover' => 'fill:{{VALUE}} !important;',
                ],
            ]
        );

        // Arrow Background Color.
        $this->add_control(
            'woolisting_arrow_bg_hover_color', [
                'label'             => esc_html__('Arrow Background Color', 'woolisting'),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-prev i:hover,
                    {{WRAPPER}} .woolisting-product-slider .slick-next i:hover' => 'background-color: {{VALUE}} !important;',
                ],
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();
        // End Hover Item Tab.

        // Arrow border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'             => 'woolisting_arrow_border',
                'selector'         => '{{WRAPPER}} .woolisting-product-slider .slick-prev i,
                {{WRAPPER}} .woolisting-product-slider .slick-next i',
            ]
        );

        // Arrow Border Radius.
        $this->add_responsive_control(
            'woolisting_arrow_border_radius', [
                'label'             => esc_html__('Arrow Border Radius', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => ['px', '%'],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-prev i,
                    {{WRAPPER}} .woolisting-product-slider .slick-next i' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        // Arrow Padding.
        $this->add_responsive_control(
            'woolisting_arrow_padding', [
                'label'             => esc_html__('Arrow Padding', 'woolisting'),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => ['px', '%'],
                'selectors'         => [
                    '{{WRAPPER}} .woolisting-product-slider .slick-prev i,
                    {{WRAPPER}} .woolisting-product-slider .slick-next i' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        // End Navigation Style.
    }

    protected function render() {

        $settings = $this->get_settings_for_display();

        $product_layout = $settings['woolisting_product_layout'];
        if ( $product_layout == 'slider' ) {
            wp_enqueue_style('slick');
            wp_enqueue_style('slick-theme');
            wp_enqueue_script('slick');
        }
        $product_style = $settings['woolisting_product_style'];
        $woolisting_slider_slides_to_show = $settings['woolisting_slider_slides_to_show'];
        $woolisting_slider_slides_to_show_tablet   =  $settings['woolisting_slider_slides_to_show_tablet'];
        $woolisting_slider_slides_to_show_mobile  = $settings['woolisting_slider_slides_to_show_mobile'];

        $args = array (
            'post_type'      => 'product',
            'post_status'    => 'publish',
            'posts_per_page' =>  $settings['woolist_number_of_show_product'],
            'order'   => $settings['woolist_category_order'],
            'orderby' => $settings['woolist_category_orderby'],
        );
        
        // Filter By Product Source.
        $args['meta_query'] = ['relation' => 'AND'];

        if ($settings['woolisting_product_source'] == 'featured-products') {
            $args['tax_query'] = [
                'relation'  => 'AND',
                [
                    'taxonomy'  => 'product_visibility',
                    'field'     => 'name',
                    'terms'     => 'featured',
                ],
                [
                    'taxonomy'  => 'product_visibility',
                    'field'     => 'name',
                    'terms'     => ['exclude-from-search', 'exclude-from-catalog'],
                    'operator'  => 'NOT IN',
                ],
            ];
        } else if ($settings['woolisting_product_source'] == 'best-selling-products') {
            $args['meta_key']   = 'total_sales';
            $args['orderby']    = 'meta_value_num';
            $args['order']      = 'DESC';
        } else if ($settings['woolisting_product_source'] == 'sale-products') {
            $args['post__in']   = array_merge( array( 0 ), wc_get_product_ids_on_sale() );
        } else if ($settings['woolisting_product_source'] == 'top-products') {
            $args['meta_key']   = '_wc_average_rating';
            $args['orderby']    = 'meta_value_num';
            $args['order']  = 'DESC';
        }

        // Exclude IN or NOT IN.
        if (isset($settings['woolist_exclude_product_categories']) && !empty($settings['woolist_exclude_product_categories'])) {
            $woolist_product_cats = 'NOT IN';
        } else {
            $woolist_product_cats = 'IN';
        }
        if (isset($settings['woolist_exclude_product_tags']) && !empty($settings['woolist_exclude_product_tags'])) {
            $woolist_product_tags = 'NOT IN';
        } else {
            $woolist_product_tags = 'IN';
        }
        
        // Query Check all Display Category and Tags.
        if ((isset($settings['woolist_product_categories']) && !empty($settings['woolist_product_categories'])) && (isset($settings['woolist_product_tags']) && !empty($settings['woolist_product_tags']))) {
            // Display All Product with Category and Tags //Condition AND.
            $args['tax_query'][] = array(
                'relation'       => 'AND',
                array(
                    'taxonomy'   => 'product_cat',
                    'field'      => 'ID',
                    'terms'      => $settings['woolist_product_categories'],
                    'operator'   => $woolist_product_cats,
                ),
                array(
                    'field'      => 'ID',
                    'taxonomy'   => 'product_tag',
                    'terms'      => $settings['woolist_product_tags'],
                    'operator'   => $woolist_product_tags,
                ),
            );
        } elseif (isset($settings['woolist_product_categories']) && !empty($settings['woolist_product_categories'])) {
            // Display All Product with Category.
            $args['tax_query'][] = array(
                array(
                    'taxonomy'   => 'product_cat',
                    'field'      => 'ID',
                    'terms'      => $settings['woolist_product_categories'],
                    'operator'   => $woolist_product_cats,
                ),
            );
        } elseif (isset($settings['woolist_product_tags']) && !empty($settings['woolist_product_tags'])) {
            // Display All Product with Tags. 
            $args['tax_query'][] = array(
                array(
                    'taxonomy'   => 'product_tag',
                    'field'      => 'ID',
                    'terms'      => $settings['woolist_product_tags'],
                    'operator'   => $woolist_product_tags,
                ),
            );
        }

        $products_query = new \WP_Query($args);

        if ($products_query) :
            $this->add_render_attribute(
                'woolisting_slider_setting', [
                    'id'                                => 'woolisting-slider-' . $this->get_id(),
                    'data-woolisting-slides-to-show'          => $woolisting_slider_slides_to_show,
                    'data-woolisting-slides-to-show-tablet'   => $woolisting_slider_slides_to_show_tablet,
                    'data-woolisting-slides-to-show-mobile'   => $woolisting_slider_slides_to_show_mobile,
                    'data-woolisting-slider-autoplay'         => $settings['woolisting_slider_autoplay'],
                    'data-woolisting-slider-autoplay-speed'   => $settings['woolisting_slider_speed'],
                    'data-woolisting-slider-dots'             => $settings['woolisting_slider_dots'],
                    'data-woolisting-slider-arrows'           => $settings['woolisting_slider_arrows'],
                    'data-woolisting-slider-to-scroll'        => $settings['woolisting_slider_slides_to_scroll'],
                    'data-woolisting-slider-infinite'         => $settings['woolisting_slider_infinite'],
                    'data-woolisting-slider-pause-on-hover'   => $settings['woolisting_slider_pause_on_hover'], 
                    'data-woolisting-prev-arrow' => Sliders_icons_managers1::render_icon($settings['woolisting_prev_icon'], [ 'aria-hidden' => 'true' ]),
                    'data-woolisting-next-arrow' => Sliders_icons_managers1::render_icon($settings['woolisting_next_icon'], [ 'aria-hidden' => 'true' ]),
                ]
            );

        $gridClass = "";
        $sliderClass = "";
        $getAttributes = "";
        if($settings['woolisting_product_layout'] == "slider") {
            $sliderClass = "woolisting-product-slider";
            $getAttributes = $this->get_render_attribute_string('woolisting_slider_setting');
        }

        if($settings['woolisting_product_layout'] == "grid") {
            $gridClass = "grid-container product-grid-gap elementor-grid " .' ';
        }
        ?>
        <div class="woolist-container <?php echo ' ' . $sliderClass. ' ' .$gridClass . ' ' . $product_style . ' ' . $product_layout; ?>elementor-grid " <?php echo $getAttributes; ?>> 

        <?php
        if(class_exists('WooCommerce')) {

            if ($products_query->have_posts()) {
                while ($products_query->have_posts()) : $products_query->the_post();
                    switch ($settings['woolisting_product_style']) {
                        case 'woolist-design-1':
                            include WOOLISTING_PATH . 'include/design/design-1.php'; // Design 1.
                            break;
                        case 'woolist-design-2':
                            include WOOLISTING_PATH . 'include/design/design-2.php'; // Design 2.
                            break;
                        case 'woolist-design-3':
                            include WOOLISTING_PATH . 'include/design/design-3.php'; // Design 3.
                            break;
                        case 'woolist-design-4':
                            include WOOLISTING_PATH . 'include/design/design-4.php'; // Design 4.
                            break;
                        default:
                        include WOOLISTING_PATH . 'include/design/design-1.php'; // Default Design 1.
                        break;      
                    }
                endwhile;
            } else {
                echo esc_html_e( 'Sorry, no products were found.', 'woolisting' );
            }
        }
        wp_reset_postdata(); 
        ?>
        </div>
        <?php
        endif;
    }
}
Plugin::instance()->widgets_manager->register(new WOOLISTING_Widget());